"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MyZodSchemaVisitor = void 0;
const graphql_1 = require("./../graphql");
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const typescript_1 = require("@graphql-codegen/typescript");
const directive_1 = require("../directive");
const importZod = `import * as myzod from 'myzod'`;
const anySchema = `definedNonNullAnySchema`;
const MyZodSchemaVisitor = (schema, config) => {
    const tsVisitor = new typescript_1.TsVisitor(schema, config);
    const importTypes = [];
    return {
        buildImports: () => {
            if (config.importFrom && importTypes.length > 0) {
                return [importZod, `import { ${importTypes.join(', ')} } from '${config.importFrom}'`];
            }
            return [importZod];
        },
        initialEmit: () => '\n' +
            [
                new visitor_plugin_common_1.DeclarationBlock({}).export().asKind('const').withName(`${anySchema}`).withContent(`myzod.object({})`)
                    .string,
            ].join('\n'),
        InputObjectTypeDefinition: {
            leave: (node) => {
                var _a;
                const name = tsVisitor.convertName(node.name.value);
                importTypes.push(name);
                const shape = (_a = node.fields) === null || _a === void 0 ? void 0 : _a.map(field => generateFieldMyZodSchema(config, tsVisitor, schema, field, 2)).join(',\n');
                return new visitor_plugin_common_1.DeclarationBlock({})
                    .export()
                    .asKind('function')
                    .withName(`${name}Schema(): myzod.Type<${name}>`)
                    .withBlock([(0, visitor_plugin_common_1.indent)(`return myzod.object({`), shape, (0, visitor_plugin_common_1.indent)('})')].join('\n')).string;
            },
        },
        ObjectTypeDefinition: {
            leave: (0, graphql_1.ObjectTypeDefinitionBuilder)(config.withObjectType, (node) => {
                var _a;
                const name = tsVisitor.convertName(node.name.value);
                importTypes.push(name);
                const shape = (_a = node.fields) === null || _a === void 0 ? void 0 : _a.map(field => generateFieldMyZodSchema(config, tsVisitor, schema, field, 2)).join(',\n');
                return new visitor_plugin_common_1.DeclarationBlock({})
                    .export()
                    .asKind('function')
                    .withName(`${name}Schema(): myzod.Type<${name}>`)
                    .withBlock([
                    (0, visitor_plugin_common_1.indent)(`return myzod.object({`),
                    (0, visitor_plugin_common_1.indent)(`__typename: myzod.literal('${node.name.value}').optional(),`, 2),
                    shape,
                    (0, visitor_plugin_common_1.indent)('})'),
                ].join('\n')).string;
            }),
        },
        EnumTypeDefinition: {
            leave: (node) => {
                var _a;
                const enumname = tsVisitor.convertName(node.name.value);
                importTypes.push(enumname);
                // z.enum are basically myzod.literals
                if (config.enumsAsTypes) {
                    return new visitor_plugin_common_1.DeclarationBlock({})
                        .export()
                        .asKind('type')
                        .withName(`${enumname}Schema`)
                        .withContent(`myzod.literals(${(_a = node.values) === null || _a === void 0 ? void 0 : _a.map(enumOption => `'${enumOption.name.value}'`).join(', ')})`)
                        .string;
                }
                return new visitor_plugin_common_1.DeclarationBlock({})
                    .export()
                    .asKind('const')
                    .withName(`${enumname}Schema`)
                    .withContent(`myzod.enum(${enumname})`).string;
            },
        },
        UnionTypeDefinition: {
            leave: (node) => {
                var _a, _b, _c;
                if (!node.types || !config.withObjectType)
                    return;
                const unionName = tsVisitor.convertName(node.name.value);
                const unionElements = (_a = node.types) === null || _a === void 0 ? void 0 : _a.map(t => {
                    var _a;
                    const element = tsVisitor.convertName(t.name.value);
                    const typ = schema.getType(t.name.value);
                    if (((_a = typ === null || typ === void 0 ? void 0 : typ.astNode) === null || _a === void 0 ? void 0 : _a.kind) === 'EnumTypeDefinition') {
                        return `${element}Schema`;
                    }
                    return `${element}Schema()`;
                }).join(', ');
                const unionElementsCount = (_c = (_b = node.types) === null || _b === void 0 ? void 0 : _b.length) !== null && _c !== void 0 ? _c : 0;
                const union = unionElementsCount > 1 ? (0, visitor_plugin_common_1.indent)(`return myzod.union([${unionElements}])`) : (0, visitor_plugin_common_1.indent)(`return ${unionElements}`);
                return new visitor_plugin_common_1.DeclarationBlock({}).export().asKind('function').withName(`${unionName}Schema()`).withBlock(union)
                    .string;
            },
        },
    };
};
exports.MyZodSchemaVisitor = MyZodSchemaVisitor;
const generateFieldMyZodSchema = (config, tsVisitor, schema, field, indentCount) => {
    const gen = generateFieldTypeMyZodSchema(config, tsVisitor, schema, field, field.type);
    return (0, visitor_plugin_common_1.indent)(`${field.name.value}: ${maybeLazy(field.type, gen)}`, indentCount);
};
const generateFieldTypeMyZodSchema = (config, tsVisitor, schema, field, type, parentType) => {
    if ((0, graphql_1.isListType)(type)) {
        const gen = generateFieldTypeMyZodSchema(config, tsVisitor, schema, field, type.type, type);
        if (!(0, graphql_1.isNonNullType)(parentType)) {
            const arrayGen = `myzod.array(${maybeLazy(type.type, gen)})`;
            const maybeLazyGen = applyDirectives(config, field, arrayGen);
            return `${maybeLazyGen}.optional().nullable()`;
        }
        return `myzod.array(${maybeLazy(type.type, gen)})`;
    }
    if ((0, graphql_1.isNonNullType)(type)) {
        const gen = generateFieldTypeMyZodSchema(config, tsVisitor, schema, field, type.type, type);
        return maybeLazy(type.type, gen);
    }
    if ((0, graphql_1.isNamedType)(type)) {
        const gen = generateNameNodeMyZodSchema(config, tsVisitor, schema, type.name);
        if ((0, graphql_1.isListType)(parentType)) {
            return `${gen}.nullable()`;
        }
        const appliedDirectivesGen = applyDirectives(config, field, gen);
        if ((0, graphql_1.isNonNullType)(parentType)) {
            if (config.notAllowEmptyString === true) {
                const tsType = tsVisitor.scalars[type.name.value];
                if (tsType === 'string')
                    return `${gen}.min(1)`;
            }
            return appliedDirectivesGen;
        }
        if ((0, graphql_1.isListType)(parentType)) {
            return `${appliedDirectivesGen}.nullable()`;
        }
        return `${appliedDirectivesGen}.optional().nullable()`;
    }
    console.warn('unhandled type:', type);
    return '';
};
const applyDirectives = (config, field, gen) => {
    if (config.directives && field.directives) {
        const formatted = (0, directive_1.formatDirectiveConfig)(config.directives);
        return gen + (0, directive_1.buildApi)(formatted, field.directives);
    }
    return gen;
};
const generateNameNodeMyZodSchema = (config, tsVisitor, schema, node) => {
    var _a, _b, _c, _d;
    const typ = schema.getType(node.value);
    if (((_a = typ === null || typ === void 0 ? void 0 : typ.astNode) === null || _a === void 0 ? void 0 : _a.kind) === 'InputObjectTypeDefinition') {
        const enumName = tsVisitor.convertName(typ.astNode.name.value);
        return `${enumName}Schema()`;
    }
    if (((_b = typ === null || typ === void 0 ? void 0 : typ.astNode) === null || _b === void 0 ? void 0 : _b.kind) === 'ObjectTypeDefinition') {
        const enumName = tsVisitor.convertName(typ.astNode.name.value);
        return `${enumName}Schema()`;
    }
    if (((_c = typ === null || typ === void 0 ? void 0 : typ.astNode) === null || _c === void 0 ? void 0 : _c.kind) === 'EnumTypeDefinition') {
        const enumName = tsVisitor.convertName(typ.astNode.name.value);
        return `${enumName}Schema`;
    }
    if (((_d = typ === null || typ === void 0 ? void 0 : typ.astNode) === null || _d === void 0 ? void 0 : _d.kind) === 'UnionTypeDefinition') {
        const enumName = tsVisitor.convertName(typ.astNode.name.value);
        return `${enumName}Schema()`;
    }
    return myzod4Scalar(config, tsVisitor, node.value);
};
const maybeLazy = (type, schema) => {
    if ((0, graphql_1.isNamedType)(type) && (0, graphql_1.isInput)(type.name.value)) {
        return `myzod.lazy(() => ${schema})`;
    }
    return schema;
};
const myzod4Scalar = (config, tsVisitor, scalarName) => {
    var _a;
    if ((_a = config.scalarSchemas) === null || _a === void 0 ? void 0 : _a[scalarName]) {
        return config.scalarSchemas[scalarName];
    }
    const tsType = tsVisitor.scalars[scalarName];
    switch (tsType) {
        case 'string':
            return `myzod.string()`;
        case 'number':
            return `myzod.number()`;
        case 'boolean':
            return `myzod.boolean()`;
    }
    console.warn('unhandled name:', scalarName);
    return anySchema;
};
